@@ -2,6 +2,7 @@ from tianshou.policy.base import BasePolicy
 from tianshou.policy.random import RandomPolicy
 from tianshou.policy.imitation.base import ImitationPolicy
 from tianshou.policy.modelfree.dqn import DQNPolicy
+from tianshou.policy.modelfree.c51 import C51Policy
 from tianshou.policy.modelfree.pg import PGPolicy
 from tianshou.policy.modelfree.a2c import A2CPolicy
 from tianshou.policy.modelfree.ddpg import DDPGPolicy
@@ -9,6 +10,7 @@ from tianshou.policy.modelfree.ppo import PPOPolicy
 from tianshou.policy.modelfree.td3 import TD3Policy
 from tianshou.policy.modelfree.sac import SACPolicy
 from tianshou.policy.modelfree.discrete_sac import DiscreteSACPolicy
+from tianshou.policy.imitation.discrete_bcq import DiscreteBCQPolicy
 from tianshou.policy.modelbase.psrl import PSRLPolicy
 from tianshou.policy.multiagent.mapolicy import MultiAgentPolicyManager
 
@@ -18,6 +20,7 @@ __all__ = [
     "RandomPolicy",
     "ImitationPolicy",
     "DQNPolicy",
+    "C51Policy",
     "PGPolicy",
     "A2CPolicy",
     "DDPGPolicy",
@@ -25,6 +28,7 @@ __all__ = [
     "TD3Policy",
     "SACPolicy",
     "DiscreteSACPolicy",
+    "DiscreteBCQPolicy",
     "PSRLPolicy",
     "MultiAgentPolicyManager",
 ]
