@@ -1,10 +1,12 @@
 from tianshou.trainer.utils import test_episode, gather_info
 from tianshou.trainer.onpolicy import onpolicy_trainer
 from tianshou.trainer.offpolicy import offpolicy_trainer
+from tianshou.trainer.offline import offline_trainer
 
 __all__ = [
     "gather_info",
     "test_episode",
     "onpolicy_trainer",
     "offpolicy_trainer",
+    "offline_trainer",
 ]
