@@ -1,7 +1,9 @@
 from tianshou.utils.config import tqdm_config
 from tianshou.utils.moving_average import MovAvg
+from tianshou.utils.log_tools import SummaryWriter
 
 __all__ = [
     "MovAvg",
     "tqdm_config",
+    "SummaryWriter",
 ]
