@@ -3,27 +3,53 @@ import numpy as np
 from torch import nn
 from typing import Any, Dict, Tuple, Union, Optional, Sequence
 
-from tianshou.data import to_torch, to_torch_as
+from tianshou.utils.net.common import MLP
+
+
+SIGMA_MIN = -20
+SIGMA_MAX = 2
 
 
 class Actor(nn.Module):
-    """Simple actor network with MLP.
+    """Simple actor network. Will create an actor operated in continuous \
+    action space with structure of preprocess_net ---> action_shape.
+
+    :param preprocess_net: a self-defined preprocess_net which output a
+        flattened hidden state.
+    :param action_shape: a sequence of int for the shape of action.
+    :param hidden_sizes: a sequence of int for constructing the MLP after
+        preprocess_net. Default to empty sequence (where the MLP now contains
+        only a single linear layer).
+    :param float max_action: the scale for the final action logits. Default to
+        1.
+    :param int preprocess_net_output_dim: the output dimension of
+        preprocess_net.
 
     For advanced usage (how to customize the network), please refer to
     :ref:`build_the_network`.
+
+    .. seealso::
+
+        Please refer to :class:`~tianshou.utils.net.common.Net` as an instance
+        of how preprocess_net is suggested to be defined.
     """
 
     def __init__(
         self,
         preprocess_net: nn.Module,
         action_shape: Sequence[int],
+        hidden_sizes: Sequence[int] = (),
         max_action: float = 1.0,
         device: Union[str, int, torch.device] = "cpu",
-        hidden_layer_size: int = 128,
+        preprocess_net_output_dim: Optional[int] = None,
     ) -> None:
         super().__init__()
+        self.device = device
         self.preprocess = preprocess_net
-        self.last = nn.Linear(hidden_layer_size, np.prod(action_shape))
+        self.output_dim = np.prod(action_shape)
+        input_dim = getattr(preprocess_net, "output_dim",
+                            preprocess_net_output_dim)
+        self.last = MLP(input_dim, self.output_dim, hidden_sizes)
         self._max = max_action
 
     def forward(
@@ -39,22 +65,40 @@ class Actor(nn.Module):
 
 
 class Critic(nn.Module):
-    """Simple critic network with MLP.
+    """Simple critic network. Will create an actor operated in continuous \
+    action space with structure of preprocess_net ---> 1(q value).
+
+    :param preprocess_net: a self-defined preprocess_net which output a
+        flattened hidden state.
+    :param hidden_sizes: a sequence of int for constructing the MLP after
+        preprocess_net. Default to empty sequence (where the MLP now contains
+        only a single linear layer).
+    :param int preprocess_net_output_dim: the output dimension of
+        preprocess_net.
 
     For advanced usage (how to customize the network), please refer to
     :ref:`build_the_network`.
+
+    .. seealso::
+
+        Please refer to :class:`~tianshou.utils.net.common.Net` as an instance
+        of how preprocess_net is suggested to be defined.
     """
 
     def __init__(
         self,
         preprocess_net: nn.Module,
+        hidden_sizes: Sequence[int] = (),
         device: Union[str, int, torch.device] = "cpu",
-        hidden_layer_size: int = 128,
+        preprocess_net_output_dim: Optional[int] = None,
     ) -> None:
         super().__init__()
         self.device = device
         self.preprocess = preprocess_net
-        self.last = nn.Linear(hidden_layer_size, 1)
+        self.output_dim = 1
+        input_dim = getattr(preprocess_net, "output_dim",
+                            preprocess_net_output_dim)
+        self.last = MLP(input_dim, 1, hidden_sizes)
 
     def forward(
         self,
@@ -63,11 +107,13 @@ class Critic(nn.Module):
         info: Dict[str, Any] = {},
     ) -> torch.Tensor:
         """Mapping: (s, a) -> logits -> Q(s, a)."""
-        s = to_torch(s, device=self.device, dtype=torch.float32)
-        s = s.flatten(1)
+        s = torch.as_tensor(
+            s, device=self.device, dtype=torch.float32  # type: ignore
+        ).flatten(1)
         if a is not None:
-            a = to_torch_as(a, s)
-            a = a.flatten(1)
+            a = torch.as_tensor(
+                a, device=self.device, dtype=torch.float32  # type: ignore
+            ).flatten(1)
             s = torch.cat([s, a], dim=1)
         logits, h = self.preprocess(s)
         logits = self.last(logits)
@@ -75,26 +121,55 @@ class Critic(nn.Module):
 
 
 class ActorProb(nn.Module):
-    """Simple actor network (output with a Gauss distribution) with MLP.
+    """Simple actor network (output with a Gauss distribution).
+
+    :param preprocess_net: a self-defined preprocess_net which output a
+        flattened hidden state.
+    :param action_shape: a sequence of int for the shape of action.
+    :param hidden_sizes: a sequence of int for constructing the MLP after
+        preprocess_net. Default to empty sequence (where the MLP now contains
+        only a single linear layer).
+    :param float max_action: the scale for the final action logits. Default to
+        1.
+    :param bool unbounded: whether to apply tanh activation on final logits.
+        Default to False.
+    :param bool conditioned_sigma: True when sigma is calculated from the
+        input, False when sigma is an independent parameter. Default to False.
+    :param int preprocess_net_output_dim: the output dimension of
+        preprocess_net.
 
     For advanced usage (how to customize the network), please refer to
     :ref:`build_the_network`.
+
+    .. seealso::
+
+        Please refer to :class:`~tianshou.utils.net.common.Net` as an instance
+        of how preprocess_net is suggested to be defined.
     """
 
     def __init__(
         self,
         preprocess_net: nn.Module,
         action_shape: Sequence[int],
+        hidden_sizes: Sequence[int] = (),
         max_action: float = 1.0,
         device: Union[str, int, torch.device] = "cpu",
         unbounded: bool = False,
-        hidden_layer_size: int = 128,
+        conditioned_sigma: bool = False,
+        preprocess_net_output_dim: Optional[int] = None,
     ) -> None:
         super().__init__()
         self.preprocess = preprocess_net
         self.device = device
-        self.mu = nn.Linear(hidden_layer_size, np.prod(action_shape))
-        self.sigma = nn.Parameter(torch.zeros(np.prod(action_shape), 1))
+        self.output_dim = np.prod(action_shape)
+        input_dim = getattr(preprocess_net, "output_dim",
+                            preprocess_net_output_dim)
+        self.mu = MLP(input_dim, self.output_dim, hidden_sizes)
+        self._c_sigma = conditioned_sigma
+        if conditioned_sigma:
+            self.sigma = MLP(input_dim, self.output_dim, hidden_sizes)
+        else:
+            self.sigma_param = nn.Parameter(torch.zeros(self.output_dim, 1))
         self._max = max_action
         self._unbounded = unbounded
 
@@ -109,9 +184,14 @@ class ActorProb(nn.Module):
         mu = self.mu(logits)
         if not self._unbounded:
             mu = self._max * torch.tanh(mu)
-        shape = [1] * len(mu.shape)
-        shape[1] = -1
-        sigma = (self.sigma.view(shape) + torch.zeros_like(mu)).exp()
+        if self._c_sigma:
+            sigma = torch.clamp(
+                self.sigma(logits), min=SIGMA_MIN, max=SIGMA_MAX
+            ).exp()
+        else:
+            shape = [1] * len(mu.shape)
+            shape[1] = -1
+            sigma = (self.sigma_param.view(shape) + torch.zeros_like(mu)).exp()
         return (mu, sigma), state
 
 
@@ -127,10 +207,11 @@ class RecurrentActorProb(nn.Module):
         layer_num: int,
         state_shape: Sequence[int],
         action_shape: Sequence[int],
+        hidden_layer_size: int = 128,
         max_action: float = 1.0,
         device: Union[str, int, torch.device] = "cpu",
         unbounded: bool = False,
-        hidden_layer_size: int = 128,
+        conditioned_sigma: bool = False,
     ) -> None:
         super().__init__()
         self.device = device
@@ -140,8 +221,13 @@ class RecurrentActorProb(nn.Module):
             num_layers=layer_num,
             batch_first=True,
         )
-        self.mu = nn.Linear(hidden_layer_size, np.prod(action_shape))
-        self.sigma = nn.Parameter(torch.zeros(np.prod(action_shape), 1))
+        output_dim = np.prod(action_shape)
+        self.mu = nn.Linear(hidden_layer_size, output_dim)
+        self._c_sigma = conditioned_sigma
+        if conditioned_sigma:
+            self.sigma = nn.Linear(hidden_layer_size, output_dim)
+        else:
+            self.sigma_param = nn.Parameter(torch.zeros(output_dim, 1))
         self._max = max_action
         self._unbounded = unbounded
 
@@ -152,7 +238,8 @@ class RecurrentActorProb(nn.Module):
         info: Dict[str, Any] = {},
     ) -> Tuple[Tuple[torch.Tensor, torch.Tensor], Dict[str, torch.Tensor]]:
         """Almost the same as :class:`~tianshou.utils.net.common.Recurrent`."""
-        s = to_torch(s, device=self.device, dtype=torch.float32)
+        s = torch.as_tensor(
+            s, device=self.device, dtype=torch.float32)  # type: ignore
         # s [bsz, len, dim] (training) or [bsz, dim] (evaluation)
         # In short, the tensor's shape in training phase is longer than which
         # in evaluation phase.
@@ -170,9 +257,14 @@ class RecurrentActorProb(nn.Module):
         mu = self.mu(logits)
         if not self._unbounded:
             mu = self._max * torch.tanh(mu)
-        shape = [1] * len(mu.shape)
-        shape[1] = -1
-        sigma = (self.sigma.view(shape) + torch.zeros_like(mu)).exp()
+        if self._c_sigma:
+            sigma = torch.clamp(
+                self.sigma(logits), min=SIGMA_MIN, max=SIGMA_MAX
+            ).exp()
+        else:
+            shape = [1] * len(mu.shape)
+            shape[1] = -1
+            sigma = (self.sigma_param.view(shape) + torch.zeros_like(mu)).exp()
         # please ensure the first dim is batch size: [bsz, len, ...]
         return (mu, sigma), {"h": h.transpose(0, 1).detach(),
                              "c": c.transpose(0, 1).detach()}
@@ -212,7 +304,8 @@ class RecurrentCritic(nn.Module):
         info: Dict[str, Any] = {},
     ) -> torch.Tensor:
         """Almost the same as :class:`~tianshou.utils.net.common.Recurrent`."""
-        s = to_torch(s, device=self.device, dtype=torch.float32)
+        s = torch.as_tensor(
+            s, device=self.device, dtype=torch.float32)  # type: ignore
         # s [bsz, len, dim] (training) or [bsz, dim] (evaluation)
         # In short, the tensor's shape in training phase is longer than which
         # in evaluation phase.
@@ -221,7 +314,8 @@ class RecurrentCritic(nn.Module):
         s, (h, c) = self.nn(s)
         s = s[:, -1]
         if a is not None:
-            a = to_torch_as(a, s)
+            a = torch.as_tensor(
+                a, device=self.device, dtype=torch.float32)  # type: ignore
             s = torch.cat([s, a], dim=1)
         s = self.fc2(s)
         return s
